@@ -3,10 +3,12 @@
  *
  * Copyright (C) 2004-2007, Clemens Fruhwirth <clemens@endorphin.org>
  * Copyright (C) 2011-2012, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2012, Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -122,7 +124,7 @@ int crypt_wipe(struct device *device,
 {
 	struct stat st;
 	char *buffer;
-	int devfd, flags, rotational, bsize;
+	int devfd, flags, bsize;
 	ssize_t written;
 
 	if (!size || size % SECTOR_SIZE || (size > MAXIMUM_WIPE_BYTES)) {
@@ -137,14 +139,12 @@ int crypt_wipe(struct device *device,
 	}
 
 	if (type == CRYPT_WIPE_DISK && S_ISBLK(st.st_mode)) {
-		rotational = 0;
-		if (!crypt_sysfs_get_rotational(major(st.st_rdev),
-						minor(st.st_rdev),
-						&rotational))
-			rotational = 1;
-		log_dbg("Rotational flag is %d.", rotational);
-		if (!rotational)
+		if (!crypt_dev_is_rotational(major(st.st_rdev),
+						minor(st.st_rdev))) {
 			type = CRYPT_WIPE_SSD;
+			log_dbg("Non-rotational device, using SSD wipe mode.");
+		} else
+			log_dbg("Rotational device, using normal wipe mode.");
 	}
 
 	bsize = device_block_size(device);
@@ -155,13 +155,14 @@ int crypt_wipe(struct device *device,
 	if (!buffer)
 		return -ENOMEM;
 
-	flags = O_RDWR | O_DIRECT | O_SYNC;
+	flags = O_RDWR;
 
 	/* use O_EXCL only for block devices */
 	if (exclusive && S_ISBLK(st.st_mode))
 		flags |= O_EXCL;
 
-	devfd = open(device_path(device), flags);
+	/* coverity[toctou] */
+	devfd = device_open(device, flags);
 	if (devfd == -1) {
 		free(buffer);
 		return errno ? -errno : -EINVAL;
